﻿using HarmonyLib;
using System;
using System.Collections;
using System.Reflection;
using UltimateWater;
using UnityEngine;
using UnityEngine.AzureSky;
using UnityEngine.UI;
using System.IO;

public class RaftOnSand : Mod
{
    public static AssetBundle asset;
    public static GameObject map;
    private readonly string harmonyId = "fr.tekgamer.raftonsand";
    private Harmony harmonyInstance;
    public static RaftOnSand instance;
    public static GameObject terrain;

    public IEnumerator Start()
    {
        instance = this;
        HNotification notification = FindObjectOfType<HNotify>().AddNotification(HNotify.NotificationType.spinning, "Loading RaftOnSand...");

        harmonyInstance = new Harmony(harmonyId);
        harmonyInstance.PatchAll(Assembly.GetExecutingAssembly());

        AssetBundleCreateRequest request = AssetBundle.LoadFromMemoryAsync(GetEmbeddedFileBytes("raftonsand.assets"));
        yield return request;

        asset = request.assetBundle;

        AssetBundleRequest terrainasset = asset.LoadAssetAsync<GameObject>("Terrain");
        yield return terrainasset;

        terrain = terrainasset.asset as GameObject;

        RaftOnSand.MainMenuBackground();

        Debug.Log("RaftOnSand has been loaded!");
        notification.Close();

    }

    public IEnumerator SetSprintSpeed()
    {
        while (RAPI.GetLocalPlayer() == null)
        {
            yield return new WaitForSeconds(1);
        }
        RAPI.GetLocalPlayer().PersonController.sprintSpeed = RAPI.GetLocalPlayer().PersonController.sprintSpeed * 2;
    }

    public static void LoadMap(Raft raft)
    {
        if (!raft.transform.Find("RaftOnSandMap"))
        {
            GameModeValueManager.GetCurrentGameModeValue().raftSpecificVariables.isRaftAlwaysAnchored = true;
            GameModeValueManager.GetCurrentGameModeValue().raftSpecificVariables.handleRaftCollision = false;
            GameModeValueManager.GetCurrentGameModeValue().raftSpecificVariables.usesBuoyancy = false;
            GameModeValueManager.GetCurrentGameModeValue().achievementVariables.trackAchievments = false;
            if (RAPI.GetLocalPlayer() != null)
            {
                RAPI.GetLocalPlayer().PersonController.sprintSpeed = RAPI.GetLocalPlayer().PersonController.sprintSpeed * 2;
            }
            else
            {
                instance.StartCoroutine(instance.SetSprintSpeed());
            }
            map = Instantiate(terrain, raft.transform);
            Water water = FindObjectOfType<Water>();
            if (water) { water.RenderingEnabled = false; }
            map.transform.localPosition = Vector3.zero;
            map.name = "RaftOnSandMap";

            Buoyancy buoyancy = FindObjectOfType<Buoyancy>();
            Transform t = (Transform)Traverse.Create(buoyancy).Field("rotatePivot").GetValue();
            t.rotation = Quaternion.Euler(0, 0, 0);
            Traverse.Create(buoyancy).Field("rotatePivot").SetValue(t);
            instance.StartCoroutine(instance.SetRaftPosition(raft));
        }
    }

    public IEnumerator SetRaftPosition(Raft raft)
    {
        if (raft == null) { yield break; }
        yield return new WaitForSeconds(2);
        if (raft == null) { yield break; }
        if (raft.transform.localPosition != new Vector3(0, 55, 0))
        {
            Traverse.Create(raft).Field("creativeTargetPos").SetValue(new Vector3(0, 55, 0));
            raft.transform.localPosition = new Vector3(0, 55, 0);
        }
        StartCoroutine(SetRaftPosition(raft));
    }

    public static void MainMenuBackground()
    {
        FindObjectOfType<AzureSkyController>().timeOfDay.GotoTime(10);
        GameObject raftLogo = GameObject.Find("Raft Logo");
        if (raftLogo != null)
        {
            raftLogo.GetComponent<Image>().sprite = RaftOnSand.asset.LoadAsset<Sprite>("raftonsand");
        }

        foreach (Camera c in FindObjectsOfType<Camera>()) { c.transform.position = c.transform.position + new Vector3(-5, 1, 1); }

        GameObject raft = GameObject.Find("Raft");
        if (raft != null && !raft.transform.Find("RaftOnSandMap"))
        {
            map = Instantiate(terrain, raft.transform);
            Water water = FindObjectOfType<Water>();
            if (water) { water.RenderingEnabled = false; }
            map.transform.localPosition = Vector3.zero;
            map.name = "RaftOnSandMap";

            FollowTransform[] followtransforms = FindObjectsOfType<FollowTransform>();
            foreach (FollowTransform ft in followtransforms)
            {
                ft.enabled = false;
            }
            Buoyancy buoyancy = FindObjectOfType<Buoyancy>();
            buoyancy.active = false;
            Transform t = (Transform)Traverse.Create(buoyancy).Field("rotatePivot").GetValue();
            t.rotation = Quaternion.Euler(0, 0, 0);
            Traverse.Create(buoyancy).Field("rotatePivot").SetValue(t);
        }
    }

    public void OnModUnload()
    {
    }
}

[HarmonyPatch(typeof(AzureSkyController))]
[HarmonyPatch("Update")]
internal class Patch_AzureSkyController_Update
{
    public static void Prefix(AzureSkyController __instance)
    {
        if (Semih_Network.InLobbyScene)
        {
            __instance.fogDistance = 500f;
        }
    }
}

[HarmonyPatch(typeof(StartMenuScreen))]
[HarmonyPatch("Start")]
public static class Patch_MainMenuBg
{
    private static void Postfix(Raft __instance)
    {
        RaftOnSand.MainMenuBackground();
    }
}

[HarmonyPatch(typeof(Raft))]
[HarmonyPatch("LateStart")]
public static class Patch_LoadingMap
{
    private static void Postfix(Raft __instance)
    {
        RaftOnSand.LoadMap(__instance);
    }
}

[HarmonyPatch(typeof(WeatherManager))]
[HarmonyPatch("StartNewWeather")]
[HarmonyPatch(new Type[] { typeof(int), typeof(bool) })]
public static class Patch_NoWeather
{
    private static bool Prefix(WeatherManager __instance, int weatherIndex, Randomizer ___weatherConnections)
    {
        return false;
    }
}